#pragma once // whatever?

#ifndef ADV_RIVALS_H
#define ADV_RIVALS_H


#include "CvEnums.h"
#include "CvPlayer.h"

class MajorCiv;


class AdvRivals {

public:

	void setInitialItems(GameSpeedTypes eSpeed);
	int computeStartTurn(int unmoddedStartTurn);
	bool oracleRefuses(int civId);
	std::wstring* forSettingsScreen();
	int barbarianStartTurn();
	int researchPercent();
	bool everShowPeaceReminder();

	// for serialization (see also note on the end of the file)
	void write(FDataStreamBase* pStream);
	void read(FDataStreamBase* pStream);
	AdvRivals();
	~AdvRivals();


private:

	void gatherCivs();
	void assignPoints();
	void assignAIPoints_xml();
	void assignAIPoints_std();
	int scaleToGrowth(int x);
	void randomizePoints();
	void bounce(int i, int j);
	void rearrangeStartingPlots();
	int dist(CvPlot* p, CvPlot* q, std::vector<CvPlot*>* startingPlot);
	int countCivsInArea(int area, std::vector<CvPlot*>* startingPlot);
	int rank(int civId);
	void reset();
	void setBarbarianStartTurn(int gameStartTurn);
	void setResearchPercent();
	void updatePointsDisplayString();
	int getFoundingPrice();
	void sortByPoints();
	int maxStartPoints();
	double meanStartPoints();
	static bool isLeftLessThanRight(MajorCiv* left, MajorCiv* right);

	// utility
	static int round(double d);
	static bool bernoulliSuccess(double pr); // 0 <= pr <= 1

	// attributes
	std::vector<MajorCiv*> civs;
	std::vector<MajorCiv*> sbp; // sorted by (start) points
	int nCivs;
	CvString report;
	CvGameSpeedInfo* speed; GameSpeedTypes eSpeed;
	MajorCiv* human;
	int m_barbarianStartTurn, m_researchPercent;
	std::wstring* pointsDisplayString;

	// for serialization
	bool randPoints;
	int prefSecluded;
	int* allPts;
};



/* wrapper for storing per-CvPlayer info (and to avoid constantly checking
   isAlive, isBarbarian etc.) */
class MajorCiv {

public:
	MajorCiv(CvPlayer& p);
	void setStartPoints_configured(int pts);
	void setStartPoints_actual(int pts);
	void assignStartPoints(); // ... to the CvPlayer
	int startPoints_configured();
	int startPoints_actual();
	bool equals(MajorCiv* other);
	CvPlot* startingPlot();
	void setStartingPlot(CvPlot* plot);
	int era();
	int id();
	bool isHuman();
	static void setFoundingPrice(int c);


private:
	int correct(int pts);

	CvPlayer& p;
	int startPoints_c, startPoints_a;
	bool actualPointsDone;
	static int foundingPrice;
};


	/* On serialization (more of a note to self):
	   Can be done in Python (and ultimately happens through Python's
	   "pickle" module) by means of CvAppInterface.py, e.g. function 'onSave'.
	   Reportedly, it's easier to serialize additional data through the DLL.
	   The design pattern appears to be:
	   * Each (entity) class has a 'write(pStream)' member function.
	     That function is supposed to pass any data to be saved to the pStream.
	     If an object has component objects, it should call those objects'
	     write functions in turn. The class that calls 'write' on the top-level
	     objects is not part of the DLL. That means, for an added class such
	     as this, the programmer needs to make sure 'write' is called from some
	     anterior 'write' function. I've made the (single) AdvRivals instance
	     a component of CvGame, and call 'write' from from 'CvGame::write'.
	   * When the player eventually loads a savegame or quits, destructors are
	     called. My AdvRivals object is an immediate member of CvGame (not 
		 just by reference), which means ~AdvRivals needn't be called explicitly,
		 same for the constructor.
	   * When the constructor is called, it could be at the start of a game,
	     or when a savegame is about to be loaded. Hence, the DLL's constructors
		 don't do much; they mostly allocate memory.
	   * Things that should only happen at the start of a game need to happen in a
	     custom function with an explicit call from e.g. CvGame::setInitialItems.
	   * When loading a savegame, read(pStream) functions are called in the same
	     manner as 'write'. 'read' needs to retrieve data in same order as they are
		 stored by 'write'.
		 As for storing complex data (i.e. objects), one way would be delegation --
		 additional read/write functions; another to extract the complex data
		 into primitive types before writing, and restore the objects from primitive data
		 in 'read'; that's what I do with the MajorCiv objects.
		 I'm uncertain, if 'read' can get called w/o prior destructor/constructor
		 calls. For one, leaving a game and starting a new one w/o quitting Civ
		 does not reinstantiate (most) objects. It seems prudent to assume that
		 both 'read' and 'setInitialItems' may or may not be preceded by
		 a constructor call, and to clear all data members before initialization;
		 --> function 'reset'.

	   I'm not fond of this pattern, especially the dummy constructors, but there's
	   no fighting it. */
#endif
