// <advri> New class; see Shelf.h for description

#include "Shelf.h"
#include "CvGameCoreDLL.h"
using std::vector;


void Shelf::add(CvPlot* plot) {

    plots.push_back(plot);
}


CvPlot* Shelf::randomPlot(int restrictionFlags, int unitDistance) {

    /* based on CvMap::syncRandPlot, but shelves are (normally) so small
       that random sampling isn't efficient. instead, compute the legal
       plots first, then return one of those at random (NULL if none) */
    vector<CvPlot*> legal;
    CvMap& map = GC.getMap();
    for(unsigned int i = 0; i < plots.size(); i++) {
        CvPlot* plot = plots[i];
        bool isLegal =
         (plot != NULL) &&
         !(RANDPLOT_LAND & restrictionFlags) &&
         (!(RANDPLOT_UNOWNED & restrictionFlags) || !plot->isOwned()) &&
         (!(RANDPLOT_ADJACENT_UNOWNED & restrictionFlags) || !plot->isAdjacentOwned()) &&
         (!(RANDPLOT_NOT_VISIBLE_TO_CIV & restrictionFlags) || !plot->isVisibleToCivTeam(true)) &&
         (!(RANDPLOT_NOT_CITY) || !plot->isCity()) && // in case a mod enables sea cities
         (!map.isAnyUnitNearby(plot, unitDistance, true));
         // RANDPLOT_PASSIBLE, RANDPLOT_ADJACENT_LAND: always satisfied
        if(isLegal)
            legal.push_back(plot);
    }
    int nLegal = legal.size();
    if(nLegal == 0)
        return NULL;
    return legal[GC.getGameINLINE().getSorenRandNum(nLegal, "advri: random shelf plot")];
}


int Shelf::nPlots() {

    return (int)plots.size();
}


/* Monitoring if plots are lost or gained would be laborious to program.
   Count them anew on every call. */
int Shelf::nUnownedPlots() {

    int r = 0;
    for(unsigned int i = 0; i < plots.size(); i++) {
        CvPlot* plot = plots[i];
        if(plot != NULL && !plot->isOwned())
            r++;
    }
    return r;
}


int Shelf::nBarbarians() {

    int r = 0;
    for(unsigned int i = 0; i < plots.size(); i++) {
        CvPlot* plot = plots[i];
        if(plot == NULL)
            continue;
		CLLNode<IDInfo>* unitNode = plot->headUnitNode();
		if(unitNode == NULL)
			continue;
        CvUnit* anyUnit = ::getUnit(unitNode->m_data);
        if(anyUnit != NULL && anyUnit->isBarbarian())
            r += plot->getNumUnits();
    }
    return r;
}


Shelf::Id::Id(int landId, int waterId) :
        std::pair<int,int>(landId, waterId) {}

// </advri>